// DragBar.h

#ifndef DragBarh
#define DragBarh

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* Usage
CDragBar implements a vertical bar which may be dragged to reposition other Child Windows.
All the Child Windows are positioned by the owner.
In the parent window create a button and subclass it.
  DragBar.SubclassDlgItem(IDC_DragBar,this);
You can now give the Bar a default position to go to when the user double-clicks it (0.0=Left, 0.5=Middle, 1.0=Right):
  DragBar.SetDefault(0.3);
If your applications other child window redraw slowly you may wish to stop them redrawing during dragging with:
  DragBar.SetLive(false);
Since you will be redrawing controls during OnSize, you need to know when OnInitDialog has created al its child windows with a bool:
  Updated=true;
Now you can set the initial positions by calling Move with no arguements:
  DragBar.Move();

The OnSize Handler would look like this:
void CDragBarTesterDlg::OnSize(UINT nType, int cx, int cy) {
	CDialog::OnSize(nType, cx, cy);
  if(!Updated) return; // Don't access windows before they've been created
  DrawControls();
}

CDragBar passes a special nType to OnSize (SIZE_BAR_DRAGGED) which you can use to filter this message if you need to:
  if(nType==SIZE_BAR_DRAGGED) DrawControls();
This would mean that you wouldn't need the "Updated" flag, and that the controls would only move when you tell them to.
Since I haven't found an event which signals the end of a resize drag, this only seems useful for non-resizing windows.

DrawControls() should use the GetClientRect and DragBar.GetPostion functions to move and redraw the controls:
  CRect Rect;
  GetClientRect(&Rect);
  int L=DragBar.GetPosition(Rect.Width()); // Left of the DragBar
  const int W=DragBar.Width+1;
  ...more sums...
  GetDlgItem(IDC_EDIT1)->MoveWindow(x,y,cx,cy);
  ...more moves...
Occasionally you'll need to Redraw a control because MoveWindow doesn't make Listboxes redraw their backgrounds etc.
  GetDlgItem(IDC_LIST1)->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW); // Don't need to erase background.
*/

#define SIZE_BAR_DRAGGED 14 // No reason - could be any number over 5. Could even add the ID of the DragBar making the call...

class CDragBar : public CWnd {
  HCURSOR Cursor;   // Mouse Pointer when over the Bar.
  double  Position; // Left of Resizing Bar as a proportion of the Dialog Client Width (also x coordinate on Resizing Bar that mouse went down on while dragging).
  double  Default;  // Default Position (restored on Double-clicking the DragBar.
  int  MinGap;      // Minimum gap before the Bar snaps to the side of the Window.
  int  Offset;      // Distance from left that Mouse Button went down (in pixels).
  bool Live;        // true when the Windows move as the user drags. If false only the Bar moves until they release the Mouse Button.
  bool Captured;    // true when the Mouse is captured
  bool Dragging;    // true when the user is dragging the window.
  void OnMove() {   // Send a SIZE_BAR_DRAGGED to the Parent's OnSize handler: 
    CRect Rect;
    GetParent()->GetClientRect(&Rect);
    GetParent()->SendMessage(WM_SIZE, SIZE_BAR_DRAGGED, MAKELONG(Rect.right, Rect.bottom));
  }
  void Capture();
  void Release();
  void Finish ();
public:
  CDragBar() : Position(0.5), Default(0.5), MinGap(2*GetSystemMetrics(SM_CXVSCROLL)), Live(true), Captured(false), Dragging(false), Cursor(AfxGetApp()->LoadStandardCursor(IDC_SIZEWE)) {}
  virtual ~CDragBar() {}
  int  GetPosition(int Width) const {return int(0.5+Position*Width);}
  void SetLive  (bool L=true)       {Live=L;} // true when the Windows move as the user drags. If false only the Bar moves until they release the Mouse Button.
  void SetMinGap(int     Gap)       {MinGap=Gap;} // Minimum gap before the Bar snaps to the side of the Window.
  void SetDefault(double Pos)       {Move(Default=Pos);} // Default Position (restored on Double-clicking the DragBar.
  void Move      (double Pos)       {Position=Pos; OnMove();} // Move to an absolute position. 0.0=Left, 0.5=Middle, 1.0=Right.
  void Move      (int   dx=0); // Move dx pixels relative to current position.
  enum {Width=11}; // Used by owner to draw their other windows.

  //{{AFX_VIRTUAL(CDragBar)
	//}}AFX_VIRTUAL
protected:
	//{{AFX_MSG(CDragBar)
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnPaint();
	afx_msg void OnCaptureChanged(CWnd *pWnd);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // ndef DragBarh
